## Normal Approximation - Classification Consistency & Accuracy ##

 ############################################
 ## YOU NEED TO SPECIFY THE FOLLOWINGS     ##
 ## rel = test reliability                 ##
 ## cut = cut score point(s)               ##
 ## data = the file that contains data     ##
 ############################################

install.packages("pbivnorm")
library(pbivnorm)

nm <- function(data, rel, cut){
  ###Define sample statistics
  n <- nrow(data)                                     # Number of examinees
  mean <- colMeans(data)                              # Observed mean
  sd <- sd(as.numeric(as.character(unlist(data))))    # SD 
  cor <- sqrt(rel)                                    # Cor b/w true and observed scores
  num.cut <- length(cut)                              # Number of cut scores
  
  ###Find z-scores for each cut score
  z <- c(1:num.cut+1)
  for(i in 1:num.cut){
    z[i] <- (cut[i]-mean)/sd
  }
  z[num.cut+1] <- 1
  
  ###Find lower probabilities for binary classification(s)
  binpl <- rep(1,num.cut+1)
  for(i in seq(num.cut)){
    binpl[i] <- pbivnorm(z[i], z[i], rho=rel)
  }
  
  ###Find upper probabilities for binary classification(s)
  binpu <- rep(0,num.cut+1)
  for(i in seq(num.cut)){
    binpu[i] <- 1-2*(pnorm(z[i])-binpl[i])-binpl[i]
  }
  
  ###Compute Agreement P for binary classification(s)
  binaryp <-rep(1,num.cut)
  for(i in seq(num.cut)){
    binaryp[i] <- binpl[i]+binpu[i]
  }
  
  ###Find chance agreement for binary classification(s) to find kappa coefficient
  pchance <-rep(1,num.cut)
  for(i in seq(num.cut)){
    pchance[i] <- pnorm(z[i])*pnorm(z[i])+(1-pnorm(z[i]))*(1-pnorm(z[i]))
  }
  
  ###Compute kappa coefficient for binary classification(s)
  binarykappa <-c(sample(1,num.cut,replace=TRUE))
  for(i in seq(num.cut)){
    binarykappa[i] <- (binaryp[i]-pchance[i])/(1-pchance[i])
  }
  
  ###Calculate accuracy (gamma) index for binary classification(s)
  binacl <-rep(1,num.cut+1)
  for(i in seq(num.cut)){
    binacl[i] <- pbivnorm(z[i],z[i]/cor, rho=cor)
  }
  
  binacu <-rep(0,num.cut+1)
  for(i in seq(num.cut)){
    binacu[i] <- 1-pnorm(z[i])-pnorm(z[i]/cor)+binacl[i]
  }
  
  binaryac <-rep(1,num.cut)
  for(i in seq(num.cut)){
    binaryac[i] <- binacl[i]+binacu[i]
  }
  
  ###False positve rate for binary classification(s)
  bin_fp <-c(rep(1,num.cut))
  
  for(i in seq(num.cut)){
    bin_fp[i] <- pnorm(z[i]/cor)-binacl[i]
  }
  
  ###False negative rate for binary classification(s)  
  bin_fn <-c(rep(1,num.cut))
  
  for(i in seq(num.cut)){
    bin_fn[i] <- pnorm(z[i])-binacl[i]
  }
  
  #####Final statistics for binary classification(s)
  binary_results <-matrix(cbind(binaryp,binarykappa,binaryac,bin_fp,bin_fn),ncol=5)
  
  k<-matrix()
  for(i in seq(num.cut)){
    k[i] <- paste("cut score",i)
  }
  
  colnames(binary_results) <- c("PHI","KAPPA","GAMMA","FALSE_POSITIVE","FALSE_NEGATIVE")
  rownames(binary_results) <- c(k)
  
  if(length(cut)>=2){
    ###Calculate the probabilities for simultaneous classification
    simul_p <-rep(1,num.cut+1)
    for(i in 2:num.cut){
      simul_p[i] <- binpl[i]-2*(pbivnorm(z[i-1],z[i],rho=rel)-binpl[i-1])-binpl[i-1]
    }
    simul_p[1] <- binpl[1]
    simul_p[num.cut+1] <- binpu[num.cut]
    
    ###Find chance agreement for simulation classification to find kappa coefficient
    simulchance <-rep(1,num.cut+1)
    for(i in seq(num.cut)){
      simulchance[i+1] <- (pnorm(z[i+1])-pnorm(z[i]))*(pnorm(z[i+1])-pnorm(z[i]))
    }
    simulchance[1] <- pnorm(z[1])*pnorm(z[1])
    simulchance[num.cut+1] <- (1-pnorm(z[num.cut]))*(1-pnorm(z[num.cut]))
    
    ###Calculate accuracy (gamma) index for simultaneous classification
    simul_ac <-rep(1,num.cut+1)
    for(i in 2:num.cut){
      simul_ac[i] <- binacl[i]-pbivnorm(z[i-1],z[i]/cor,rho=cor)-pbivnorm(z[i-1]/cor,z[i],rho=cor)+binacl[i-1]
    }
    simul_ac[1] <- binacl[1]
    simul_ac[num.cut+1] <- binacu[num.cut]
    
    ###False positve rate for simultaneous classification
    simul_fp <-rep(0,num.cut)
    
    simul_fp[1] <- pnorm(z[1]/cor)-binacl[1]
    
    for(i in 2:num.cut){
      simul_fp[i] <- pnorm(z[i]/cor)-binacl[i]-(pnorm(z[i-1]/cor)-pbivnorm(z[i-1]/cor,z[i],rho=cor))
    }
    
    ###False negative rate for simultaneous classification
    simul_fn <-rep(1,num.cut)
    
    simul_fn[1] <- pnorm(z[1])-binacl[1]
    
    for(i in 2:num.cut){
      simul_fn[i] <- pnorm(z[i])-binacl[i]-(pnorm(z[i-1])-pbivnorm(z[i-1],z[i]/cor,rho=cor))
    }
    
    #####Final statistics for simulataneous classification
    PHI <- sum(simul_p)
    simulc <- sum(simulchance)
    KAPPA <- (PHI-simulc)/(1-simulc)
    GAMMA <- sum(simul_ac)
    FL.POS <- sum(simul_fp)
    FL.NEG <- sum(simul_fn)
    
    simultaneous <- matrix(cbind(PHI, KAPPA, GAMMA, FL.POS, FL.NEG),ncol=5)
    colnames(simultaneous) <- c("PHI","KAPPA","GAMMA","FALSE_POSITIVE","FALSE_NEGATIVE")
    rownames(simultaneous) <- paste(num.cut+1,"categories")
    
    final <- list(binary_results,simultaneous)
    names(final) <- c("Binary Classifications", "Simultaneous Classification")
    print(final)
  } else {
    final <- list(binary_results)
    names(final) <- c("Binary Classification")
    print(final)
  }
}
